require 'pl'

-- Convert a decimal number into its binary representation
function dec2bin(dec)
   if dec>0 then
      return tostring(dec2bin(math.floor(dec/2)))..tostring(dec%2)
   else
      return ""
   end
end

-- Convert Decimal to Binary and pad for alignment
-- dec: Decimal number
-- num: binary length padding (8,16,32 etc..)
-- return: a string of binary for the number
function Dec2Bin(dec,num)
   return stringx.ljust(dec2bin(dec),num,'0')
end

-- Binary to Decimal
function bin2dec(num,weight)
   if num==0 then
      return 0
   else
      return (num%2)*math.pow(2,weight) + bin2dec(math.floor(num/10), weight+1)
   end
end

-- Binary to Decimal
function Bin2Dec(num)
   return bin2dec(num,0)
end

-- Convert a string of space separated decimal numbers into a binary stream
function stream2bin(stream)
   return table.concat(tablex.map(function(x) return Dec2Bin(x,8):reverse() end, 
				  tablex.map(tonumber, utils.split(stream))))
end

-- Flip all bits of a binary string (1's Complement)
function onesComplement(b)
   return b:gsub("([01])",function(x) return utils.choose(x=="0","1","0") end)
end

-- Take a binary string and slice them into integers on bit boundaries.
-- binary: a binary stream
-- ...: list bits to extract for next int value (negative to sign-extend)
-- Returns the list of extracted integer values.
function bitslicer(binary, ...)
   local pos = 1
   local t={}
   local n,b
   for k,v in ipairs{...} do
      n = math.abs(v)
      b = binary:sub(pos, pos+n-1):reverse()
      -- Handle -ve bit slices with sign-extension.
      if v < 0 and b:sub(1,1) == "1" then
	 -- 2's Complement and flip sign.
	 t[k] = -(Bin2Dec(onesComplement(b))+1)
      else
	 t[k] = Bin2Dec(b)
      end
      pos = pos + n
   end
   return unpack(t)
end

--[[
Equivalent in C speak to converting the ASCII stream to its 
equiv byte stream and then copying this into the payload struct.

struct {
    byte light;     // light sensor: 0..255
    byte moved :1;  // motion detector: 0..1
    byte humi  :7;  // humidity: 0..100
    int temp   :10; // temperature: -500..+500 (tenths)
    byte lobat :1;  // supply voltage dropped under 3.1V: 0..1
} payload;

unsigned char stream[]={0,0,12,2};
memcpy(&payload, stream, sizeof(stream));
--]]

stream="0 0 12 2"
light, moved, humi, temp, lobat = bitslicer(stream2bin(stream),8,1,7,-10,1)

print("Light: "..light)
print("Moved: "..moved)
print("humi: "..humi)
print("temp: "..temp/10)
print("lobat: "..lobat)
