-- LUA parser for an XAP message

msg=[[
xap-header
{
v=13
hop=1
uid=FF.9192:00
class=xAPBSC.event
source=UKUSA.MOBOTIX.MX12USA
}
output.state
{
state=Off
text=92220
}]]

function decodeXap(msg)
   local line

   local nextLine = coroutine.wrap(function(msg)
				      if not msg:find '\n$' then msg = msg..'\n' end
				      for i in msg:gfind("([^\n]*)\n") do
					 coroutine.yield(i)
				      end
				end)

   local function readBody()
     local _,k,v,t
     line = nextLine()
     t={}
     while line ~= nil and line ~= "}" do
       _,_,k,v = line:find("(.-)%s*=%s*(.*)")
       t[k:lower()] = v
       line = nextLine()
     end
     return t
   end

   local function readXap(msg)
     local t,section
     t={}
     line = nextLine(msg)

     while line ~= nil do
       section = line
       nextLine() -- Skip the {
       t[section:lower()] = readBody()
       line = nextLine()
     end
     return t
   end

   return readXap(msg)
end

require("pl")
tbl = decodeXap(msg)
pretty.dump(tbl)

--[[
{
{
  ["xap-header"] = {
    uid = "FF.9192:00", 
    source = "UKUSA.MOBOTIX.MX12USA", 
    hop = "1", 
    class = "xAPBSC.event", 
    v = "13", 
  },
  ["output.state"] = {
    state = "Off", 
    text = "92220", 
  }
}
--]]

