class = require "pl.class".class


class.Frame()

MSG_ORDINARY = "xap-header"
MSG_HBEAT = "xap-hbeat"
MSG_UNKNOWN = "unknown"

function Frame:_init(msg)
   local line

   local nextLine = coroutine.wrap(function(msg)
     for i in msg:gfind("(.-)\n") do
       coroutine.yield(i)
     end
     coroutine.yield(nil)
   end)

   local function readBody()
     local _,k,v,t
     line = nextLine()
     t={}
     while line ~= nil and line ~= "}" do
       _,_,k,v = line:find("(.-)%s*=%s*(.*)")
       t[k:lower()] = v
       line = nextLine()
     end
     return t
   end

   local function readXap(msg)
     local t,section
     t={}
     line = nextLine(msg)
     while line ~= nil do
       section = line
       nextLine() -- Skip the {
       t[section:lower()] = readBody()
       line = nextLine()
     end
     return t
   end
   
   return readXap(msg)
end

function Frame:getValue(section, key)
  if self[section] ~= nil then
    if key == nil then
      return "XAP_FILTER_ANY"
    end
    return self[section][key]
  else
    return nil
  end
end

function Frame:isValue(section, key, value)
  return self:getValue(section, key) == value
end

function Frame:getType()
  if self["xap-header"] ~= nil then return MSG_ORDINARY end
  if self["xap-hbeat"] ~= nil then return MSG_HBEAT end
  return MSG_UNKNOWN
end

function Frame:__tostring()
  local function encodeSection(s)
    local t,v
    t = s..'\n{\n'
    for k,v in pairs(self[s]) do
      t = t..k.."="..v.."\n"
    end
    t = t..'}\n'
    return t
  end

  local header = 'xap-header'
  if self[header] == nil then type = 'xap-hbeat' end    
  local out = encodeSection(header)
  for k,_ in pairs(self) do
    if k ~= header then
      out = out .. encodeSection(k)
    end
  end

  return out
end
